<?php

namespace Drupal\feeds_tamper_server_files\Plugin\Tamper;

use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\file\Entity\File;
use Drupal\tamper\Exception\TamperException;
use Drupal\tamper\TamperableItemInterface;
use Drupal\tamper\TamperBase;
use Drupal\file\FileInterface;

/**
 * Plugin implementation for converting server file paths to Drupal file entities.
 *
 * @Tamper(
 *   id = "server_file_path",
 *   label = @Translation("Server File Path to File Entity"),
 *   description = @Translation("Converts a server file path to a Drupal file entity."),
 *   category = "Files and Images"
 * )
 */
class ServerFilePathTamper extends TamperBase {

  const SETTING_SOURCE_DIRECTORY = 'source_directory';
  const SETTING_DESTINATION_DIRECTORY = 'destination_directory';
  const SETTING_FILE_EXISTS = 'file_exists';
  const SETTING_MAKE_PERMANENT = 'make_permanent';
  const SETTING_ALLOWED_EXTENSIONS = 'allowed_extensions';

  /**
   * {@inheritdoc}
   */
  public function tamper($data, TamperableItemInterface $item = NULL) {
    if (empty($data)) {
      return '';
    }

    try {
      // Check if the data is already a file ID
      if (is_numeric($data)) {
        $file = File::load($data);
        if ($file) {
          return $data; // Return the existing file ID
        }
      }

      $file_system = \Drupal::service('file_system');
      
      // Clean and validate the input path
      $source_path = trim($data);
      
      // Get the original filename with extension
      $filename = basename($source_path);
      
      // Validate file extension
      $extension = pathinfo($filename, PATHINFO_EXTENSION);
      if (empty($extension)) {
        throw new TamperException(sprintf('No file extension found in filename: %s', $filename));
      }

      // Check against allowed extensions
      $allowed_extensions = explode(' ', strtolower($this->getSetting(self::SETTING_ALLOWED_EXTENSIONS)));
      if (!in_array(strtolower($extension), $allowed_extensions)) {
        throw new TamperException(sprintf(
          'File extension %s is not allowed. Allowed extensions are: %s',
          $extension,
          implode(', ', $allowed_extensions)
        ));
      }

      // Prepare and validate source file path
      $source_directory = $this->getSetting(self::SETTING_SOURCE_DIRECTORY);
      $source_uri = rtrim($source_directory, '/') . '/' . $filename;

      // Convert source URI to real path and check if file exists
      $source_real_path = $file_system->realpath($source_uri);
      if (!$source_real_path || !file_exists($source_real_path)) {
        throw new TamperException(sprintf('Source file not found at: %s', $source_uri));
      }

      // Prepare destination
      $destination_directory = rtrim($this->getSetting(self::SETTING_DESTINATION_DIRECTORY), '/');
      $destination_uri = $destination_directory . '/' . $filename;

      // Ensure destination directory exists
      if (!$file_system->prepareDirectory($destination_directory, FileSystemInterface::CREATE_DIRECTORY)) {
        throw new TamperException(sprintf('Could not create destination directory: %s', $destination_directory));
      }

      // Check for existing file by URI
      $existing_files = \Drupal::entityTypeManager()
        ->getStorage('file')
        ->loadByProperties(['uri' => $destination_uri]);
      
      if ($existing_file = reset($existing_files)) {
        return $existing_file->id();
      }

      // Copy file
      $copied_uri = $file_system->copy(
        $source_real_path,
        $destination_uri,
        $this->getSetting(self::SETTING_FILE_EXISTS)
      );

      if (!$copied_uri) {
        throw new TamperException(sprintf('Failed to copy file from %s to %s', $source_real_path, $destination_uri));
      }

      // Create file entity
      $mime_guesser = \Drupal::service('file.mime_type.guesser');
      $file = File::create([
        'uid' => \Drupal::currentUser()->id(),
        'filename' => basename($copied_uri),
        'uri' => $copied_uri,
        'status' => $this->getSetting(self::SETTING_MAKE_PERMANENT) ? FileInterface::STATUS_PERMANENT : FileInterface::STATUS_TEMPORARY,
        'filemime' => $mime_guesser->guessMimeType($copied_uri),
      ]);
      
      $file->save();
      return $file->id();

    }
    catch (\Exception $e) {
      \Drupal::logger('feeds_tamper_server_files')->error(
        'Server File Path Tamper error: @message',
        ['@message' => $e->getMessage()]
      );
      throw new TamperException($e->getMessage());
    }
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    $config = parent::defaultConfiguration();
    $config[self::SETTING_SOURCE_DIRECTORY] = 'public://import_source';
    $config[self::SETTING_DESTINATION_DIRECTORY] = 'public://imported_files';
    $config[self::SETTING_FILE_EXISTS] = FileSystemInterface::EXISTS_RENAME;
    $config[self::SETTING_MAKE_PERMANENT] = TRUE;
    $config[self::SETTING_ALLOWED_EXTENSIONS] = 'png gif jpg jpeg pdf doc docx';
    return $config;
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form[self::SETTING_SOURCE_DIRECTORY] = [
      '#type' => 'textfield',
      '#title' => $this->t('Source Directory'),
      '#description' => $this->t('Base directory for source files (e.g., public://import_source or /var/www/html/web/sites/default/import_source/).'),
      '#default_value' => $this->getSetting(self::SETTING_SOURCE_DIRECTORY),
      '#required' => TRUE,
    ];

    $form[self::SETTING_DESTINATION_DIRECTORY] = [
      '#type' => 'textfield',
      '#title' => $this->t('Destination Directory'),
      '#description' => $this->t('Directory where files will be copied (e.g., public://imported_files).'),
      '#default_value' => $this->getSetting(self::SETTING_DESTINATION_DIRECTORY),
      '#required' => TRUE,
    ];

    $form[self::SETTING_ALLOWED_EXTENSIONS] = [
      '#type' => 'textfield',
      '#title' => $this->t('Allowed file extensions'),
      '#description' => $this->t('Space-separated list of allowed file extensions (e.g., "png gif jpg jpeg").'),
      '#default_value' => $this->getSetting(self::SETTING_ALLOWED_EXTENSIONS),
      '#required' => TRUE,
    ];

    $form[self::SETTING_FILE_EXISTS] = [
      '#type' => 'select',
      '#title' => $this->t('Existing files'),
      '#options' => [
        FileSystemInterface::EXISTS_REPLACE => $this->t('Replace'),
        FileSystemInterface::EXISTS_RENAME => $this->t('Rename'),
        FileSystemInterface::EXISTS_ERROR => $this->t('Error'),
      ],
      '#default_value' => $this->getSetting(self::SETTING_FILE_EXISTS),
    ];

    $form[self::SETTING_MAKE_PERMANENT] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Make files permanent'),
      '#description' => $this->t('If checked, imported files will be marked as permanent.'),
      '#default_value' => $this->getSetting(self::SETTING_MAKE_PERMANENT),
    ];

    return $form;
  }
}
